.cropper-container {
    background-color: #fff;
    overflow: hidden;
    position: relative;
    -webkit-tap-highlight-color: transparent;
    -webkit-touch-callout: none;
    -moz-user-select: none;
    -webkit-user-select: none;
    -ms-user-select: none;
    user-select: none;
}
.cropper-container .cropper-modal {
    background: #000;
    height: 100%;
    left: 0;
    opacity: 0.5;
    filter: alpha(opacity=50);
    position: absolute;
    top: 0;
    width: 100%;
}
.cropper-container .cropper-dragger {
    height: 80%;
    left: 10%;
    position: absolute;
    top: 10%;
    width: 80%;
}
.cropper-container .cropper-preview {
    border-width: 1px;
    border-style: solid;
    border-color: #36f;
    border-color: rgba(51, 102, 255, 0.8);
    -moz-box-sizing: border-box;
    box-sizing: border-box;
    display: block;
    height: 100%;
    overflow: hidden;
    width: 100%;
}
.cropper-container .cropper-dashed {
    background-color: #36f;
    display: block;
    font-size: 0;
    height: 100%;
    line-height: 0;
    overflow: hidden;
    position: absolute;
    width: 100%;
    opacity: 0.5;
    filter: alpha(opacity=50);
}
.cropper-container .dashed-h {
    background-color: transparent;
    border-bottom: 1px dashed #fff;
    border-top: 1px dashed #fff;
    left: 0;
    top: 33.3%;
    height: 33.3%;
}
.cropper-container .dashed-v {
    background-color: transparent;
    border-left: 1px dashed #fff;
    border-right: 1px dashed #fff;
    left: 33.3%;
    top: 0;
    width: 33.3%;
}
.cropper-container .cropper-face,
.cropper-container .cropper-line,
.cropper-container .cropper-point {
    display: block;
    font-size: 0;
    height: 100%;
    line-height: 0;
    overflow: hidden;
    position: absolute;
    width: 100%;
}
.cropper-container .cropper-face {
    background-color: #fff;
    cursor: move;
    left: 0;
    opacity: 0.1;
    filter: alpha(opacity=10);
    top: 0;
}
.cropper-container .cropper-line {
    background-color: #36f;
    opacity: 0.1;
    filter: alpha(opacity=10);
}
.cropper-container .line-e {
    cursor: e-resize;
    right: -2px;
    top: 0;
    width: 4px;
}
.cropper-container .line-n {
    cursor: n-resize;
    height: 4px;
    left: 0;
    top: -2px;
}
.cropper-container .line-w {
    cursor: w-resize;
    left: -2px;
    top: 0;
    width: 4px;
}
.cropper-container .line-s {
    cursor: s-resize;
    bottom: -2px;
    height: 4px;
    left: 0;
}
.cropper-container .cropper-point {
    background-color: #36f;
    height: 4px;
    width: 4px;
    opacity: 0.8;
    filter: alpha(opacity=80);
}
.cropper-container .point-e {
    cursor: e-resize;
    right: -2px;
    top: 49%;
}
.cropper-container .point-n {
    cursor: n-resize;
    left: 49%;
    top: -2px;
}
.cropper-container .point-w {
    cursor: w-resize;
    left: -2px;
    top: 49%;
}
.cropper-container .point-s {
    cursor: s-resize;
    bottom: -2px;
    left: 49%;
}
.cropper-container .point-ne {
    cursor: ne-resize;
    right: -2px;
    top: -2px;
}
.cropper-container .point-nw {
    cursor: nw-resize;
    left: -2px;
    top: -2px;
}
.cropper-container .point-sw {
    cursor: sw-resize;
    bottom: -2px;
    left: -2px;
}
.cropper-container .point-se {
    cursor: se-resize;
    bottom: -2px;
    right: -2px;
}
.cropper-hidden {
    display: none !important;
}